import { ISignal } from '../core/signaling';
/**
 * An object which holds data related to a widget's title.
 *
 * #### Notes
 * A title object is intended to hold the data necessary to display a
 * header for a particular widget. A common example is the `TabPanel`,
 * which uses the widget title to populate the tab for a child widget.
 */
export declare class Title {
    /**
     * Construct a new title.
     *
     * @param options - The options for initializing the title.
     */
    constructor(options?: Title.IOptions);
    /**
     * A signal emitted when the state of the title changes.
     */
    changed: ISignal<Title, void>;
    /**
     * Get the object which owns the title.
     *
     * #### Notes
     * This will be `null` if the title has no owner.
     *
     * This is a read-only property.
     */
    readonly owner: any;
    /**
     * Get the label for the title.
     *
     * #### Notes
     * The default value is an empty string.
     */
    /**
     * Set the label for the title.
     */
    label: string;
    /**
     * Get the mnemonic index for the title.
     *
     * #### Notes
     * The default value is `-1`.
     */
    /**
     * Set the mnemonic index for the title.
     */
    mnemonic: number;
    /**
     * Get the icon class name for the title.
     *
     * #### Notes
     * The default value is an empty string.
     */
    /**
     * Set the icon class name for the title.
     *
     * #### Notes
     * Multiple class names can be separated with whitespace.
     */
    icon: string;
    /**
     * Get the caption for the title.
     *
     * #### Notes
     * The default value is an empty string.
     */
    /**
     * Set the caption for the title.
     */
    caption: string;
    /**
     * Get the extra class name for the title.
     *
     * #### Notes
     * The default value is an empty string.
     */
    /**
     * Set the extra class name for the title.
     *
     * #### Notes
     * Multiple class names can be separated with whitespace.
     */
    className: string;
    /**
     * Get the closable state for the title.
     *
     * #### Notes
     * The default value is `false`.
     */
    /**
     * Set the closable state for the title.
     *
     * #### Notes
     * This controls the presence of a close icon when applicable.
     */
    closable: boolean;
    private _label;
    private _icon;
    private _caption;
    private _mnemonic;
    private _className;
    private _closable;
    private _owner;
}
/**
 * The namespace for the `Title` class statics.
 */
export declare namespace Title {
    /**
     * An options object for initializing a title.
     */
    interface IOptions {
        /**
         * The object which owns the title.
         */
        owner?: any;
        /**
         * The label for the title.
         */
        label?: string;
        /**
         * The mnemonic index for the title.
         */
        mnemonic?: number;
        /**
         * The icon class for the title.
         */
        icon?: string;
        /**
         * The caption for the title.
         */
        caption?: string;
        /**
         * The extra class name for the title.
         */
        className?: string;
        /**
         * The closable state for the title.
         */
        closable?: boolean;
    }
}
